==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/fir/analysis/jvm/checkers/declaration/FirJvmRedundantRepeatableChecker.kt <==
/*
 * Copyright 2010-2022 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.jvm.checkers.declaration

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirBasicDeclarationChecker
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.jvm.FirJvmErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.declarations.getAnnotationByClassId
import ksp.org.jetbrains.kotlin.fir.expressions.unexpandedClassId
import ksp.org.jetbrains.kotlin.name.FqName
import ksp.org.jetbrains.kotlin.name.JvmStandardClassIds
import ksp.org.jetbrains.kotlin.name.JvmStandardClassIds.Annotations.JvmRepeatable
import ksp.org.jetbrains.kotlin.name.StandardClassIds.Annotations.Repeatable

object FirJvmRedundantRepeatableChecker : FirBasicDeclarationChecker(MppCheckerKind.Common) {
    override fun check(declaration: FirDeclaration, context: CheckerContext, reporter: DiagnosticReporter) {
        val kotlinRepeatable = declaration.getAnnotationByClassId(Repeatable, context.session)
        val javaRepeatable = declaration.getAnnotationByClassId(JvmRepeatable, context.session)
            ?: declaration.getAnnotationByClassId(JvmStandardClassIds.Annotations.Java.Repeatable, context.session)

        if (kotlinRepeatable != null && javaRepeatable != null) {
            reporter.reportOn(
                kotlinRepeatable.source,
                FirJvmErrors.REDUNDANT_REPEATABLE_ANNOTATION,
                kotlinRepeatable.unexpandedClassId?.asSingleFqName() ?: FqName.ROOT,
                javaRepeatable.unexpandedClassId?.asSingleFqName() ?: FqName.ROOT,
                context
            )
        }
    }
}

==> symbol-processing-aa-embeddable-2.3.2-sources.jar: org/jetbrains/kotlin/fir/analysis/jvm/checkers/declaration/FirJvmRedundantRepeatableChecker.kt <==
/*
 * Copyright 2010-2022 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.jvm.checkers.declaration

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.diagnostics.reportOn
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirBasicDeclarationChecker
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.jvm.FirJvmErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.declarations.getAnnotationByClassId
import ksp.org.jetbrains.kotlin.fir.expressions.unexpandedClassId
import ksp.org.jetbrains.kotlin.name.FqName
import ksp.org.jetbrains.kotlin.name.JvmStandardClassIds
import ksp.org.jetbrains.kotlin.name.JvmStandardClassIds.Annotations.JvmRepeatable
import ksp.org.jetbrains.kotlin.name.StandardClassIds.Annotations.Repeatable

object FirJvmRedundantRepeatableChecker : FirBasicDeclarationChecker(MppCheckerKind.Common) {
    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(declaration: FirDeclaration) {
        val kotlinRepeatable = declaration.getAnnotationByClassId(Repeatable, context.session)
        val javaRepeatable = declaration.getAnnotationByClassId(JvmRepeatable, context.session)
            ?: declaration.getAnnotationByClassId(JvmStandardClassIds.Annotations.Java.Repeatable, context.session)

        if (kotlinRepeatable != null && javaRepeatable != null) {
            reporter.reportOn(
                kotlinRepeatable.source,
                FirJvmErrors.REDUNDANT_REPEATABLE_ANNOTATION,
                kotlinRepeatable.unexpandedClassId?.asSingleFqName() ?: FqName.ROOT,
                javaRepeatable.unexpandedClassId?.asSingleFqName() ?: FqName.ROOT
            )
        }
    }
}
